/**
 * Copies any text passed to the parameter with showing confirmation message in case of success
 * or showing an prompt for handling the copy manually if the browser doesn't support auto copy
 *
 * @returns {boolean}
 *
 * @param {string} text -- The string to be copied
 * @param {Function} i18n -- The translation function for translating the shown confirmation message - eg. plugin.i18n
 * @param {Function} info -- The filerobot info function for showing confirmation message - eg. plugin.filerobot.info
 * @param {string} successMsgI18nKey -- (optional) The translation key of the success msg that will show after copying text, default is 'File link is copied'
 */
export default function copyText(_ref) {
  var text = _ref.text,
    i18n = _ref.i18n,
    info = _ref.info,
    successMsgI18nKey = _ref.successMsgI18nKey,
    _ref$status = _ref.status,
    status = _ref$status === void 0 ? 'info' : _ref$status,
    _ref$disableSuccessMs = _ref.disableSuccessMsg,
    disableSuccessMsg = _ref$disableSuccessMs === void 0 ? false : _ref$disableSuccessMs;
  navigator.clipboard.writeText(text);
  if (!disableSuccessMsg) {
    info(i18n(successMsgI18nKey || 'mutualizedFileLinkCopiedInfo'), status);
  }
}