/**
 * Takes a file object and turns it into fileID, by converting file.name to lowercase,
 * removing extra characters and adding type, size and lastModified
 *
 * @param {object} file
 * @returns {string} the fileID
 */
export default function generateFileId(file) {
  // It's tempting to do `[items].filter(Boolean).join('-')` here, but that
  // is slower! simple string concatenation is fast

  var id = 'filerobot';
  if (typeof file.name === 'string') {
    id += '-' + encodeFilename(file.name.toLowerCase());
  }
  if (file.toFolder !== undefined) {
    id += '-' + encodeFilename(file.toFolder.toLowerCase());
  }
  if (file.type !== undefined) {
    id += '-' + file.type;
  }
  if (file.data) {
    if (typeof file.data.relativePath === 'string') {
      id += '-' + encodeFilename(file.data.relativePath.toLowerCase());
    }

    /**
     * TODO: check the following comment after the commiting time,
     * Removed the data size, and last modified, to make the id unified for the files
     * so if a new version is being uploaded we will override the base on in the progress panel's activities if there
     * If no issues reported about progress panel/activities related to this comment after some time, please remove it and improve the wrapper condition.
     * related to this task: https://scaleflexhq.atlassian.net/browse/FRA-7291
    */

    // if (file.data.size !== undefined) {
    //   id += '-' + file.data.size
    // }
    // if (file.data.lastModified !== undefined) {
    //   id += '-' + file.data.lastModified
    // }
  }
  return id;
}
function encodeFilename(name) {
  var suffix = '';
  return name.replace(/[^A-Z0-9]/ig, function (character) {
    suffix += '-' + encodeCharacter(character);
    return '/';
  }) + suffix;
}
function encodeCharacter(character) {
  return character.charCodeAt(0).toString(32);
}