function _toConsumableArray(r) { return _arrayWithoutHoles(r) || _iterableToArray(r) || _unsupportedIterableToArray(r) || _nonIterableSpread(); }
function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(r, a) { if (r) { if ("string" == typeof r) return _arrayLikeToArray(r, a); var t = {}.toString.call(r).slice(8, -1); return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray(r, a) : void 0; } }
function _iterableToArray(r) { if ("undefined" != typeof Symbol && null != r[Symbol.iterator] || null != r["@@iterator"]) return Array.from(r); }
function _arrayWithoutHoles(r) { if (Array.isArray(r)) return _arrayLikeToArray(r); }
function _arrayLikeToArray(r, a) { (null == a || a > r.length) && (a = r.length); for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e]; return n; }
import { TIME_OPTIONS } from './constants';
import getItemIcon from './getItemIcon';
import getFileExtension from './getFileExtension';
import prettyFileSize from './prettyFileSize';
import getFileType from './getFileType';
import humanReadableDateWithTranslations from './humanReadableDateWithTranslations';
import Translator from './Translator';
import { getActualVisibility, getVisibilityI18nLabel } from './visibility';
export default (function (file, i18n) {
  var _fileInfo$img_w, _fileInfo$img_h, _getFileExtension, _file$owner, _fileInfo$brand, _file$folder;
  var fileInfo = (file === null || file === void 0 ? void 0 : file.info) || {};
  var width = (_fileInfo$img_w = fileInfo.img_w) !== null && _fileInfo$img_w !== void 0 ? _fileInfo$img_w : fileInfo.video_w;
  var height = (_fileInfo$img_h = fileInfo.img_h) !== null && _fileInfo$img_h !== void 0 ? _fileInfo$img_h : fileInfo.video_h;
  var isDefined = width && height;
  var type = getFileType(file);
  var extension = ((_getFileExtension = getFileExtension(file)) === null || _getFileExtension === void 0 ? void 0 : _getFileExtension.toUpperCase()) || '';
  var videoDuration = fileInfo.video_duration;
  var transcodingProgress = file.transcodingProgress;
  var uuid = file.uuid;
  var name = file.name;
  var urls = file.url;
  var preview = file.preview || getItemIcon({
    fileOrFolder: file
  });
  var createdAt = file.created_at ? new Date(file.created_at).toLocaleTimeString(Translator.LANGUAGE, TIME_OPTIONS) : '';
  var hasI18n = typeof i18n === 'function';
  var humanReadableCreatedAt = hasI18n ? humanReadableDateWithTranslations(file.created_at, i18n) : createdAt;
  var createdBy = (_file$owner = file.owner) === null || _file$owner === void 0 ? void 0 : _file$owner.name;
  var modifiedAt = file.modified_at || file.updated_at ? new Date(file.modified_at).toLocaleTimeString(Translator.LANGUAGE, TIME_OPTIONS) : '';
  var humanReadableModifiedAt = hasI18n ? humanReadableDateWithTranslations(file.modified_at, i18n) : modifiedAt;
  var ratio = isDefined ? (width / height).toPrecision(3) : 0;
  var size = prettyFileSize(file.size);
  var tags = file !== null && file !== void 0 && file.tags ? file.tags : {};
  var resolution = isDefined ? "".concat(width, " \xD7 ").concat(height) : null;
  var actualVisibility = getActualVisibility(file.visibility);
  var visibility = hasI18n ? i18n(getVisibilityI18nLabel(actualVisibility)) : actualVisibility;
  var brandsAndLogos = (((_fileInfo$brand = fileInfo.brand) === null || _fileInfo$brand === void 0 ? void 0 : _fileInfo$brand.extracted) || []).map(function (item) {
    return item === null || item === void 0 ? void 0 : item.classes;
  });
  var brandAndLogosLabels = brandsAndLogos.map(function (item) {
    var _item$;
    return item === null || item === void 0 ? void 0 : (_item$ = item[0]) === null || _item$ === void 0 ? void 0 : _item$.label;
  });
  var filteredBrandAndLogosLabels = _toConsumableArray(new Set(brandAndLogosLabels));
  var meta = file.meta || {};
  var folderPath = (_file$folder = file.folder) === null || _file$folder === void 0 ? void 0 : _file$folder.name;
  var colorMode = fileInfo.color_space;
  return {
    uuid: uuid,
    name: name,
    type: type,
    extension: extension,
    size: size,
    createdAt: createdAt,
    humanReadableCreatedAt: humanReadableCreatedAt,
    createdBy: createdBy,
    modifiedAt: modifiedAt,
    humanReadableModifiedAt: humanReadableModifiedAt,
    info: fileInfo,
    width: width,
    height: height,
    urls: urls,
    ratio: ratio,
    resolution: resolution,
    tags: tags,
    visibility: visibility,
    meta: meta,
    preview: preview,
    videoDuration: videoDuration,
    transcodingProgress: transcodingProgress,
    brandAndLogosLabels: filteredBrandAndLogosLabels,
    folderPath: folderPath,
    colorMode: colorMode
  };
});