import { isStaticSelectOption } from '@filerobot/explorer/lib/utils/filters';
import { FILTER_CONDITIONS } from './constants';
import { getFieldValueQuery } from './fieldsQueriesGetters';
var createStaticValueIsQuery = function createStaticValueIsQuery(key, value) {
  return "".concat(key, ":").concat(value.join(','));
};
var createIsQuery = function createIsQuery(key, value) {
  if (isStaticSelectOption(value)) {
    return createStaticValueIsQuery(key, value);
  }
  return "".concat(key, ":=").concat(value.join(','));
};
var createIsNotQuery = function createIsNotQuery(key, value) {
  return value.map(function (item) {
    return "".concat(key, ":-").concat(item);
  }).join(' ');
};
var createContainsQuery = function createContainsQuery(metadataFilter, value) {
  return getFieldValueQuery(metadataFilter, value);
};
var createContainsNotQuery = function createContainsNotQuery(_ref) {
  var metadataFilter = _ref.metadataFilter,
    key = _ref.key,
    value = _ref.value,
    extraValue = _ref.extraValue;
  var valueQuery = getFieldValueQuery(metadataFilter, value);
  var extraValueQuery = extraValue.map(function (item) {
    return "".concat(key, ":-").concat(item);
  }).join(' ');
  return "".concat(valueQuery, " ").concat(extraValueQuery).trim();
};

/**
 * Generates a multi-select field query based on the metadata filter.
*  Docs: https://scaleflexhq.atlassian.net/wiki/spaces/FIL/pages/466452497
 *
 * @param {Object} metadataFilter - The metadata filter object.
 * @returns {string} - The resulting query string.
 */

export var getMetadataMultiSelectQuery = function getMetadataMultiSelectQuery(metadataFilter) {
  var key = metadataFilter.key,
    _metadataFilter$value = metadataFilter.value,
    value = _metadataFilter$value === void 0 ? [] : _metadataFilter$value,
    _metadataFilter$condi = metadataFilter.condition,
    condition = _metadataFilter$condi === void 0 ? FILTER_CONDITIONS.IS : _metadataFilter$condi,
    _metadataFilter$extra = metadataFilter.extraValue,
    extraValue = _metadataFilter$extra === void 0 ? [] : _metadataFilter$extra;
  switch (condition) {
    case FILTER_CONDITIONS.IS:
      return createIsQuery(key, value);
    case FILTER_CONDITIONS.IS_NOT:
      return createIsNotQuery(key, value);
    case FILTER_CONDITIONS.CONTAINS:
      return createContainsQuery(metadataFilter, value.join(','));
    case FILTER_CONDITIONS.CONTAINS_NOT:
      return createContainsNotQuery({
        metadataFilter: metadataFilter,
        key: key,
        value: value.join(','),
        extraValue: extraValue
      });
    default:
      return '';
  }
};