import getGeoPointValidationErrors from './getGeoPointValidationErrors';
import isValidUri from '../isValidUri';

/**
 * Check is metadata field value valid
 * @param {(string|number|array)} value
 * @param {string} [fieldValidation]
 * @param {boolean} [required]
 * @returns {(boolean|string)} - If TRUE field is valid, if FALSE or message field is not valid.
 * @param {boolean} [canEdit=true]
 */
function checkFieldValidation(_ref) {
  var fieldValue = _ref.fieldValue,
    fieldValidation = _ref.fieldValidation,
    isFieldRequired = _ref.isFieldRequired,
    _ref$editable = _ref.editable,
    editable = _ref$editable === void 0 ? true : _ref$editable,
    i18n = _ref.i18n,
    fieldType = _ref.fieldType;
  if (isFieldRequired && editable && (!fieldValue || Array.isArray(fieldValue) && !fieldValue.length)) {
    return typeof i18n === 'function' ? i18n('mutualizedFieldRequired') : 'Field is required!';
  }

  /**
   * When field is required but current user can't edit the field, we have to set that current value is valid to unlock other fields save.
   */
  if (isFieldRequired && !editable) {
    return true;
  }
  if (fieldType === 'attachment-uri' && !isValidUri(fieldValue) && fieldValue) {
    return false;
  }
  if (fieldType === 'geopoint' && Object.keys(getGeoPointValidationErrors({
    geoPoint: fieldValue
  })).length > 0 && fieldValue) {
    return false;
  }
  if (!fieldValidation) {
    // If no fieldValidation rules return TRUE (field valid)
    return true;
  }
  if (typeof fieldValidation !== 'string') {
    console.warn('Metadata field validation: support string only!', {
      fieldValue: fieldValue,
      fieldValidation: fieldValidation
    });
    return true;
  }
  return new RegExp(fieldValidation).test(fieldValue) ? true : "String \"".concat(fieldValue, "\" not match regexp ").concat(fieldValidation);
}
export default checkFieldValidation;