import checkFieldValidation from './checkFieldValidation';

/**
 * @param {object} errors
 * @param {string} key
 * @param {string} [regionalVariantFieldKey]
 * @param {any} fieldValue
 * @param {regexp} [fieldValidation]
 * @param {boolean} [isFieldRequired]
 * @param {(boolean|string[])} [canEdit=true]
 */
function updateValidationErrors(_ref) {
  var errors = _ref.errors,
    key = _ref.key,
    regionalVariantFieldKey = _ref.regionalVariantFieldKey,
    fieldValue = _ref.fieldValue,
    fieldValidation = _ref.fieldValidation,
    isFieldRequired = _ref.isFieldRequired,
    _ref$canEdit = _ref.canEdit,
    canEdit = _ref$canEdit === void 0 ? true : _ref$canEdit,
    i18n = _ref.i18n,
    fieldType = _ref.fieldType;
  var errorsClone = JSON.parse(JSON.stringify(errors || {}));
  var editable = canEdit === true || regionalVariantFieldKey && Array.isArray(canEdit) && canEdit.indexOf(regionalVariantFieldKey) > -1;
  var isValid = checkFieldValidation({
    fieldValue: fieldValue,
    fieldValidation: fieldValidation,
    isFieldRequired: isFieldRequired,
    editable: editable,
    i18n: i18n,
    fieldType: fieldType
  });
  if (isValid === true) {
    // Valid, remove error
    if (regionalVariantFieldKey) {
      if (errorsClone[key] && Object.prototype.hasOwnProperty.call(errorsClone[key], regionalVariantFieldKey)) {
        delete errorsClone[key][regionalVariantFieldKey];
        if (!Object.keys(errorsClone[key]).length) {
          // Object empty, need to remove
          delete errorsClone[key];
        }
      }
    } else {
      if (Object.prototype.hasOwnProperty.call(errorsClone, key)) {
        delete errorsClone[key];
      }
    }
  } else {
    // Not valid, add error
    if (regionalVariantFieldKey) {
      errorsClone[key] = errorsClone[key] || {};
      errorsClone[key][regionalVariantFieldKey] = isValid || true;
    } else {
      errorsClone[key] = isValid || true;
    }
  }
  return errorsClone;
}
export default updateValidationErrors;