/**
 * Removes duplicate objects from an array based on a specified unique property.
 *
 * This function iterates through the array and uses a Set to keep track of the unique values
 * of the specified property. If a duplicate value is found, the object is filtered out.
 *
 * @param {Array<Object>} array - The array of objects to be filtered.
 * @param {string} uniqueProperty - The property name to check for uniqueness.
 * @returns {Array<Object>} A new array with duplicate objects removed.
 */

var removeDuplicatesFromArray = function removeDuplicatesFromArray(array, uniqueProperty) {
  var seenSet = new Set();
  return array.filter(function (item) {
    var duplicate = seenSet.has(item[uniqueProperty]);
    seenSet.add(item[uniqueProperty]);
    return !duplicate;
  });
};
export default removeDuplicatesFromArray;